#include <opengl_drawer.hpp>

#include <mesh.hpp>

#include <iostream>
#include <sstream>
#include <GL/glu.h>

namespace cpe
{
  void opengl_drawer::draw(const mesh& m)
  {
      unsigned int N_tri=m.number_of_triangle();
      const int* p_connectivity=&m.get_connectivity()[0];
      const double* p_vertex=m.get_vertices()[0].pointer();
      const double* p_normal=m.get_normal()[0].pointer();

      if(m.get_normal().size()!=m.get_vertices().size())
      {
          std::stringstream msg;
          msg<<"Error, incorrect normals for this mesh "<<std::endl;
          msg<<"Number of vertices: ["<<m.get_vertices().size()<<"] / Number of normales ["<<m.get_normal().size()<<"]";
          if(m.get_normal().size()==0)
              msg<<std::endl<<"(Use mesh.compute_normal() before drawing)"<<std::endl;
          throw cpe::exception_cpe(msg.str(),__FILE__,__FUNCTION__,__LINE__);
      }


      //enable client state
      glEnableClientState(GL_VERTEX_ARRAY);
      glEnableClientState(GL_NORMAL_ARRAY);

      //setup pointers
      glVertexPointer(3,GL_DOUBLE,0,p_vertex);
      glNormalPointer(GL_DOUBLE,0,p_normal);

      //draw triangles
      glDrawElements(GL_TRIANGLES,3*N_tri,GL_UNSIGNED_INT,p_connectivity);

      //disable client state
      glDisableClientState(GL_VERTEX_ARRAY);
      glDisableClientState(GL_NORMAL_ARRAY);

  }
}
