#include "monWidgetGL.hpp"

#include <mesh.hpp>
#include <opengl_drawer.hpp>
#include <matrix3.hpp>

#include <GL/glu.h>
#include <libmesh/mesh.hpp>
#include <libmesh/mesh_io.hpp>
#include <iostream>

using cpe::v2;
using cpe::v3;
using cpe::mesh;

monWidgetGL::monWidgetGL(QGLWidget *parent) :
    QGLWidget(parent)
{
    QWidget::setFocusPolicy(Qt::StrongFocus);
    etat_affichage=true;


    translation=v3(0,0,0);
    double angle=0.0;
}

monWidgetGL::~monWidgetGL()
{}

void monWidgetGL::initializeGL()
{
    std::cout<<"appel monWidgetGL::initializeGL()"<<std::endl;

    glClearColor(1.0, 1.0, 1.0, 0.0);

    //chargement de maillage
    mesh_0.load_file("data/camel.off");

    //rotation et translation du maillage
    mesh_0.apply(cpe::matrix3::rotation(v3(1,0,0),-M_PI/2.0));
    mesh_0.apply(cpe::matrix3::rotation(v3(0,1,0),3*M_PI/4.0));
    mesh_0.auto_scale(1.0);
    mesh_0+=v3(0,0.8,-7);

    //calcul des normales par sommets
    mesh_0.compute_normal();




    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);

    //light
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glFrontFace( GL_CW ) ;
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    float light_position[4]={1.0,-8.0,-5.0,float(1.0)};
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glEnable(GL_COLOR_MATERIAL);

}

void monWidgetGL::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);



    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60,1.0,0.1,50.0);
    glRotated(180.0*angle/M_PI,0,1,0);
    glTranslated(translation.x(),translation.y(),translation.z());


    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();




    if(etat_affichage==true)
    {
        glColor3d(0.8,0.7,0.6);
        cpe::opengl_drawer::draw(mesh_0);


        glNormal3d(0,-1,0);
        glColor3d(0.9,0.9,0.9);
        glBegin(GL_QUADS);
        glVertex3d(-8,-0.2,-8);
        glVertex3d( 8,-0.2,-8);
        glVertex3d( 8,-0.2, 8);
        glVertex3d(-8,-0.2, 8);
        glEnd();

    }


}



void monWidgetGL::afficher()
{
    etat_affichage=!etat_affichage;

    updateGL();
}
void monWidgetGL::wireframe(bool est_actif)
{
    if(est_actif==true)
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);

    updateGL();
}

void monWidgetGL::keyPressEvent(QKeyEvent *event)
{

    double d_angle=0.05;
    double d_radial=0.1;
    double d_height=0.1;

    v3 d_tr(-d_radial*sin(angle),0,d_radial*cos(angle));
    int current=event->key();


    switch(current)
    {
    case Qt::Key_Up:
        translation += d_tr;
        break;
    case Qt::Key_Down:
        translation -= d_tr;
        break;
    case Qt::Key_Left:
        angle-=d_angle;
        break;
    case Qt::Key_Right:
        angle+=d_angle;
        break;

    case Qt::Key_A:
        translation.y() -= d_height;
        break;
    case Qt::Key_Q:
        translation.y() += d_height;
        break;
    }

    updateGL();
}


