#include "monWidgetGL.hpp"

#include <mesh.hpp>
#include <opengl_drawer.hpp>
#include <matrix3.hpp>

#include <GL/glu.h>
#include <libmesh/mesh.hpp>
#include <libmesh/mesh_io.hpp>
#include <iostream>

using cpe::v2;
using cpe::v3;
using cpe::mesh;

monWidgetGL::monWidgetGL(QGLWidget *parent) :
    QGLWidget(parent)
{
    QWidget::setFocusPolicy(Qt::StrongFocus);
    etat_affichage=true;

    startTimer(25);

    texture_id=0;
    texture_id_ground=0;
    translation=v3(0,0,0);
    angle=0.0;
}

monWidgetGL::~monWidgetGL()
{}

void monWidgetGL::initializeGL()
{
    std::cout<<"appel monWidgetGL::initializeGL()"<<std::endl;

    glClearColor(1.0, 1.0, 1.0, 0.0);

    //chargement de maillage
    mesh_0.load_file("data/camel.off");
    mesh_0.load_file("data/cat.obj");

    //rotation et translation du maillage
    mesh_0.apply(cpe::matrix3::rotation(v3(0,1,0),M_PI/4.0));
    mesh_0.auto_scale(1.0);
    mesh_0+=v3(0,0.8,-8);


    chargement_terrain();

    mesh_sphere.load_file("data/sphere.off");
    mesh_sphere.auto_scale();
    mesh_sphere*=0.25;
    mesh_sphere+=v3(2,0,-8);
    mesh_sphere.compute_normal();
    mesh_sphere_original=mesh_sphere;

    mesh_ground.compute_normal();

    texture_id=bindTexture(QString("data/cat.png"),GL_TEXTURE_2D);
    if(texture_id==0)
        std::cout<<"Chargement texture echouee"<<std::endl;


    //calcul des normales par sommets
    try
    {
        mesh_0.compute_normal();
    }
    catch(cpe::exception_cpe e)
    {
        std::cout<<"Erreur dans le calcul des normales"<<std::endl;
        std::cout<<e.info()<<std::endl;
        exit(0);
    }




    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);

    //light
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glFrontFace( GL_CW ) ;
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    float light_position[4]={1.0,-8.0,-5.0,float(1.0)};
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glEnable(GL_COLOR_MATERIAL);

}

void monWidgetGL::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);



    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60,1.0,0.1,50.0);
    glRotated(180.0*angle/M_PI,0,1,0);
    glTranslated(translation.x(),translation.y(),translation.z());


    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();




    if(etat_affichage==true)
    {
        glColor3d(1,1,1);

        glEnable(GL_TEXTURE_2D);




        glBindTexture(GL_TEXTURE_2D,texture_id);
        cpe::opengl_drawer::draw(mesh_0);

        glBindTexture(GL_TEXTURE_2D,texture_id_ground);
        cpe::opengl_drawer::draw(mesh_ground);


        glDisable(GL_TEXTURE_2D);
        glColor3d(0.9,0.4,0.4);
        update_mesh_sphere();
        cpe::opengl_drawer::draw(mesh_sphere);


    }


}



void monWidgetGL::afficher()
{
    etat_affichage=!etat_affichage;

    updateGL();
}
void monWidgetGL::wireframe(bool est_actif)
{
    if(est_actif==true)
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);

    updateGL();
}

void monWidgetGL::keyPressEvent(QKeyEvent *event)
{

    double d_angle=0.05;
    double d_radial=0.1;
    double d_height=0.1;

    v3 d_tr(-d_radial*sin(angle),0,d_radial*cos(angle));
    int current=event->key();


    switch(current)
    {
    case Qt::Key_Up:
        translation += d_tr;
        break;
    case Qt::Key_Down:
        translation -= d_tr;
        break;
    case Qt::Key_Left:
        angle-=d_angle;
        break;
    case Qt::Key_Right:
        angle+=d_angle;
        break;

    case Qt::Key_A:
        translation.y() -= d_height;
        break;
    case Qt::Key_Q:
        translation.y() += d_height;
        break;
    }

    updateGL();
}


void monWidgetGL::chargement_terrain()
{
    mesh_ground.add_vertex(v3(-10,-0.2,-10));
    mesh_ground.add_vertex(v3(-10,-0.2, 10));
    mesh_ground.add_vertex(v3( 10,-0.2, 10));
    mesh_ground.add_vertex(v3( 10,-0.2,-10));
    mesh_ground.add_triangle(0,1,2);
    mesh_ground.add_triangle(0,2,3);
    mesh_ground.add_texture(v2(0,0));
    mesh_ground.add_texture(v2(5,0));
    mesh_ground.add_texture(v2(5,5));
    mesh_ground.add_texture(v2(0,5));
    texture_id_ground=bindTexture(QString("data/seamless_grass_texture.png"),GL_TEXTURE_2D);
    if(texture_id_ground==0)
        std::cout<<"Chargement texture echouee"<<std::endl;
}

void monWidgetGL::update_mesh_sphere()
{
    double time=static_cast<double>(compteur_temps)/1000.0;
    mesh_sphere=mesh_sphere_original+v3(0,0.1+1+sin(9*2*M_PI*time),0);
}

void monWidgetGL::timerEvent(QTimerEvent *event)
{
    compteur_temps=(compteur_temps+1)%1000;


    event->accept();
    updateGL();


}
